% random_numbers_tictoc
% script to demonstrate the effects of loops and pre-defining matrices
% on computation time.
% uses tic and toc to compare computation time of several ways to generate
% a matrix filled with random numbers.
% This script is part of the Matlab course for PhD Students 2010-2011:
% http://www.sinnesphysiologie.uni-oldenburg.de/50246.html
% written by Jutta Kretzberg, 07.01.2011

disp('compare different solutions to create a 300x300 random number matrix')


%% Buildt-in
disp('Buildt-in function rand')
tic
M1 = rand(300, 300);
toc


%% for loop
disp('for-loop without pre-defined matrix')
tic
for x=1:300*300
    M2(x) = rand();
end
M2=reshape(M2,300,300);
toc

%% for-loop with pre-defined matrix
disp('for-loop with pre-defined matrix')
tic
M3=zeros(300,300);  % saves a lot of time because JIT can be used
for x=1:300*300
    M3(x) = rand();
end
toc

%% for-loop concatinating rows
disp('for-loop concatinating rows')
tic
for x=1:300
    M4(x,:)=randn(1,300);
end
toc


%% nested for-loops without pre-defined matrix
disp('nested for-loops without pre-defined matrix')
tic
for x=1:300
    for y=1:300
        M5(x,y) = rand();
    end
end
toc

%% nested for-loops and pre-defined matrix
disp('nested for-loops and pre-defined matrix')
tic
M6 = zeros(300, 300); %saves a lot of computation time, because of JIT
for x=1:300
    for y=1:300
        M6(x,y) = rand();
    end
end
toc

%% while-loop and reshape
disp('while-loop and reshape')
tic
M7 = [];
x = 1;
while x<=300*300
    M7 = [M7 rand()];
    x = x+1;
end
toc
tic
M7 = reshape(M7,300, 300);
toc

%% While-loop and pre-defined matrix
tic
disp('While-loop and pre-defined matrix')
M8 = zeros(300);
X=1;
while X <300^2
    M8(X)=rand(1);
    X=X+1;
end
toc

