function Counts=updown2a(Nums)
% function Counts=updown2a(Nums)
% up-down algorithm: Let N be a positive integer. If N is even, divide it
% by 2. But if it is odd, multiply it by 3 and add 1. Repeat until N 
% becomes 1. 
% Optimized version, doing two steps together for odd numbers.
% Input: vector of positive integer numbers Nums
% Output: Counts is the number of repetitions needed for each element of 
% Nums to converge to 1
% Modified from the book Mastering Matlab 7
% Modified by Jutta Kretzberg for Matlab course for PhD students, 
% WS 2010/2011: http://www.sinnesphysiologie.uni-oldenburg.de/50246.html

%% test function input:
% Nums should be a vector of positive integers
if ~exist('Nums','var') || ~isnumeric(Nums) || ~isvector(Nums)  
   warning('Input Nums should be a vector of positive integer values. Set Nums=1:25.') 
   Nums = 1:25;
end
% find indices of negative or not integer values
indi=find(Nums~=round(Nums)|Nums<1, 1);
if ~isempty(indi)
  warning('Replace negative or non-integer values of Nums by positive integers')
  Nums(indi)=abs(round(Nums(indi)));  
end

%% lengths of up-down sequences for each vector element
for z=1:length(Nums)
   N = Nums(z);  % number to test
   count = 0;  % iteration count
	while N>1
       if rem(N,2)==0 % even
          N = N/2;
          count = count+1;
       else                     % odd numbers: Optimization: since the number
                            % must be even in the next step, do the two
                            % steps together.
          N = (3*N+1)/2;
          count = count+2;
       end
	end
	Counts(z) = count;
end
