function Counts=updown_vector_opt(Nums)
% function Counts=updown_vector(Nums)
% up-down algorithm: Let N be a positive integer. If N is even, divide it
% by 2. But if it is odd, multiply it by 3 and add 1. Repeat until N 
% becomes 1. 
% Vector version, doing steps for all odd numbers and all even numbers in 
% Nums simultaneously.
% Optimized by using fix instead of rem function.
% Input: vector of positive integer numbers Nums
% Output: Counts is the number of repetitions needed for each element of 
% Nums to converge to 1
% Modified from the book Mastering Matlab 7
% Modified by Jutta Kretzberg for Matlab course for PhD students, 
% WS 2010/2011: http://www.sinnesphysiologie.uni-oldenburg.de/50246.html

%% test function input:
% Nums should be a vector of positive integers
if ~exist('Nums','var') || ~isnumeric(Nums) || ~isvector(Nums)  
   warning('Input Nums should be a vector of positive integer values. Set Nums=1:25.') 
   Nums = 1:25;
end
% find indices of negative or not integer values
indi=find(Nums~=round(Nums)|Nums<1, 1);
if ~isempty(indi)
  warning('Replace negative or non-integer values of Nums by positive integers')
  Nums(indi)=abs(round(Nums(indi)));  
end



%% pre-define variables:
N = Nums;                % duplicate numbers
Counts = zeros(size(N)); % preallocate array

not1 = N>1;              % True for numbers greater than one



%% lengths of up-down sequences for all vector elements together

while any(not1)                 % true if any element of vector not1 is a nonzero element
   
   odd = (N-2*fix(N/2))~=0;      % Vector with 1 for odd values and 0 for even,
                                 % without using rem function
   
   odd_not1 = odd & not1;        % Vector with 1 for odd values greater than one
   even_not1 = ~odd & not1;      % Vector with 1 for even values greater than one
   
   N(even_not1) = N(even_not1)/2;           % Process evens
   Counts(even_not1) = Counts(even_not1)+1;
   
   N(odd_not1) = (3*N(odd_not1)+1)/2;       % Process odds, next two steps 
                                            % together like in updown2a 
   Counts(odd_not1) = Counts(odd_not1)+2;
   
   not1 = N>1;                   % Find remaining numbers
end